#                       Copyright 2021 Saad Mairaj
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import tkinter
from tkmacosx.utils import (SYSTEM_DEFAULT, _cnfmerge, _bind, check_param)


MARQUEE_PROPERTIES = [
    'background', 'bg', 'borderwidth', 'bd', 'cursor', 'disabledforeground',
    'end_delay', 'foreground', 'font', 'fps', 'height', 'highlightbackground',
    'highlightcolor', 'highlightthickness', 'initial_delay', 'justify',
    'left_margin', 'relief', 'smoothness', 'state', 'takefocus', 'text', 'width',
]


class MarqueeBase(tkinter.Canvas):
    """Base class for Marquee."""

    def __init__(self, master=None, cnf={}, **kw):
        kw = _cnfmerge((cnf, kw))
        self._stop_state = False
        self.after_id = ' '
        cnf = dict(
            text=kw.pop('text', ''),
            font=kw.pop('font', 'TkDefaultFont'),
            fg=kw.pop('fg', kw.pop('foreground', SYSTEM_DEFAULT.fg)),
            fps=kw.pop('fps', 30),
            justify=kw.pop('justify', 'left'),
            left_margin=kw.pop('left_margin', 10),
            initial_delay=kw.pop('initial_delay', 1000),
            end_delay=kw.pop('end_delay', 1000),
            smoothness=kw.pop('smoothness', 1),  # 1 <= smooth < 1
            disabledforeground=kw.pop('disabledforeground', 'grey')
        )
        cnf['foreground'] = cnf['fg']
        cnf = {k: v for k, v in cnf.items() if v is not None}

        kw['bg'] = kw.get('bg', kw.get('background', SYSTEM_DEFAULT.bg))
        kw['height'] = kw.get('height', 24)
        kw['highlightthickness'] = kw.get('highlightthickness', 0)
        tkinter.Canvas.__init__(self, master=master, **kw)
        self.cnf = check_param(self, 'marquee', **cnf)
        self._set_text()
        _bind(self, className='configure',
              sequence='<Configure>', func=self._check)

    def _set_text(self, reset=False):
        TAG = 'text'
        itemconfig = lambda kw={}: tkinter.Canvas._configure(
            self, ('itemconfigure', TAG), None, kw)
        if reset or not self.find_withtag(TAG):
            COORD = (3, self.winfo_height()/2)
            ANCHOR = 'w'
            self.delete(TAG)
            self._create(
                'text', COORD, dict(anchor=ANCHOR, tag=TAG))

        # Configure text item
        for op in list(itemconfig()):
            if (op in self.cnf and
                    not ('foreground' in op or 'fg' in op)):
                itemconfig({op: self.cnf[op]})

        self.cnf['font'] = self.itemcget(TAG, 'font')
        self.cnf['justify'] = self.itemcget(TAG, 'justify')

        itemconfig(
            dict(state=self['state'],
                 disabledfill=self.cnf.get('disabledforeground'),
                 fill=self.cnf.get('foreground'))
        )

    def _set_height(self, evt=None):
        """Internal function."""
        bbox = self.bbox('text')
        height = bbox[3] - bbox[1] + 8
        if int(self['height']) == height:
            return
        tkinter.Canvas._configure(
            self, 'configure', {'height': height}, None)

    def _reset(self, force_reset=False):
        """Internal function.\n
        Resets the text position, do not call directly."""
        if self.after_id == ' ' and not force_reset:
            return
        self.after_cancel(self.after_id)
        self.coords('text', 3, self.winfo_height()/2)  # RESETS TEXT
        self.after_id = ' '

    def _check(self, evt=None):
        """Internal function.\n
        Sets the text properly in the frame."""
        self._set_height()
        self.coords('text', 3, self.winfo_height()/2)
        text_width = self.bbox('text')[2]   # TEXT WIDTH
        frame_width = self.winfo_width()    # FRAME WIDTH
        if text_width + 1 < frame_width:
            self._reset()
        elif self.after_id == ' ':
            delay = self.cnf.get('initial_delay')  # INITITAL DELAY
            self.after_id = self.after(delay, self._animate)

    def _animate(self, evt=None):
        """Internal function.\n
        Process text and move text."""
        if self._stop_state:
            return
        self._set_height()
        text_width = self.bbox('text')[2]   # TEXT WIDTH
        frame_width = self.winfo_width()    # FRAME WIDTH
        delay = int(self.cnf.get('smoothness')*1000 / self.cnf.get('fps'))
        if text_width + 1 + self.cnf.get('left_margin') < frame_width:
            self.after(self.cnf.get('end_delay'), self.coords,
                       'text', 3, self.winfo_height()/2)  # RESETS TEXT
            delay = self.cnf.get(
                'initial_delay') + self.cnf.get(
                    'end_delay')  # INITITAL DELAY
        else:
            # MOVE -1 PIXEL EVERYTIME
            self.tk.call((self._w, 'move') + (
                'text', -self.cnf.get('smoothness'), 0))
        self.after_id = self.after(delay, self._animate)

    def _configure(self, cmd, cnf, kw):
        """Internal function."""
        kw = _cnfmerge((cnf, kw))
        diff = set(self.keys()).difference(
            list(tkinter.Canvas._configure(
                self, cmd, None, None)))
        for i in diff:
            if i in kw:
                self.cnf.update(check_param(
                    self, 'marquee', {i: kw.pop(i)}))
        self._set_text()
        self._set_height()
        values = tkinter.Canvas._configure(self, cmd, None, kw)
        if values is not None and isinstance(values, dict):
            values.update(self.cnf.copy())
            for i in list(values.copy()):
                if i not in MARQUEE_PROPERTIES:
                    values.pop(i)
        return values

    def keys(self):
        """Return a list of all resource names of this widget."""
        return MARQUEE_PROPERTIES

    def cget(self, key):
        """Return the resource value for a KEY given as string."""
        if key in self.cnf:
            val = self.cnf[key]
            if (('foreground' in key or 'fg' in key)
                    and isinstance(val, tkinter.Variable)):
                return val.get()
            return str(val)
        return tkinter.Canvas.cget(self, key)
    __getitem__ = cget

    def destroy(self):
        """Destroy this widget."""
        self.after_cancel(self.after_id)
        return tkinter.Canvas.destroy(self)



#                       Copyright 2021 Saad Mairaj
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from tkmacosx.basewidgets.button_base import ButtonBase


class CircleButton(ButtonBase):
    """Circle button widget."""

    def __init__(self, master=None, cnf={}, **kw):
        """
        Circle shaped Button supports almost all options of a tkinter button 
        and have some few more useful options such as 'activebackground', overbackground', 
        'overforeground', 'activeimage', 'activeforeground', 'borderless' and much more. 

        To check all the configurable options for CircleButton run `print(CircleButton().keys())`.

        Example:
        ```
        import tkinter as tk
        import tkmacosx as tkm

        root = tk.Tk()
        cb = tkm.CircleButton(root, text='Hello')
        cb.pack()
        root.mainloop()
        ```
        """
        ButtonBase.__init__(self, 'circle', master, cnf, **kw)



#                       Copyright 2021 Saad Mairaj
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from tkmacosx.basewidgets.button_base import ButtonBase


class Button(ButtonBase):
    """Button widget."""

    def __init__(self, master=None, cnf={}, **kw):
        """Button for macos, supports almost all the features of tkinter button,

            - There are few extra features as compared to default Tkinter Button:
            - To check the list of all the resources. To get an overview about
                the allowed keyword arguments call the method `keys`. 
                    print(Button().keys())

        ### Examples:
        ```
        import tkinter as tk
        import tkmacosx as tkm
        import tkinter.ttk as ttk

        root = tk.Tk()
        root.geometry('200x200')
        tkm.Button(root, text='Mac OSX', bg='lightblue', fg='yellow').pack()
        tk.Button(root, text='Mac OSX', bg='lightblue', fg='yellow').pack()
        ttk.Button(root, text='Mac OSX').pack()
        root.mainloop()
        ```

        ### Get a cool gradient effect in activebackground color.
        ```
        import tkinter as tk
        import tkmacosx as tkm

        root = tk.Tk()
        root.geometry('200x200')
        tkm.Button(root, text='Press Me!!', activebackground=('pink','blue') ).pack()
        tkm.Button(root, text='Press Me!!', activebackground=('yellow','green') ).pack()
        tkm.Button(root, text='Press Me!!', activebackground=('red','blue') ).pack()
        root.mainloop()
        ```"""
        ButtonBase.__init__(self, 'normal', master, cnf, **kw)



#                       Copyright 2021 Saad Mairaj
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from tkmacosx.basewidgets.sframe_base import SFrameBase


class SFrame(SFrameBase):
    "Sframe widget. (Scrollable frame)"

    def __init__(self, master=None, cnf={}, **kw):
        """### Scrollable Frame ButtonBase.    
        (Only supports vertical scrolling)

        Sames as tkinter Frame. These are some extra resources.
        - `scrollbarwidth`: Set the width of scrollbar.
        - `mousewheel`: Set mousewheel scrolling.
        - `avoidmousewheel`: Give widgets that also have mousewheel scrolling and is a child of SFrame \
            this will configure widgets to support their mousewheel scrolling as well. \
            For eg:- Text widget inside SFrame can have mousewheel scrolling as well as SFrame.

        Scrollbar of SFrame can be configured by calling `scrollbar_configure(**options)`. 
        To access methods of the scrollbar it can be called through the scrollbar instance `self['scrollbar']`.

        ### How to use?
        Use it like a normal frame.

        ### Example:
        ```
        root = Tk()
        frame = SFrame(root, bg='pink')
        frame.pack()

        for i in range(100):
            Button(frame, text='Button %s'%i).pack()

        root.mainloop()
        ```"""
        SFrameBase.__init__(self, master=master, cnf=cnf, **kw)
        self.scrollbar_configure = self['scrollbar'].configure



#                       Copyright 2021 Saad Mairaj
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""
This module provides some modified widgets of Tkinter which works better on macos
and some more useful functions and classes as well. For example Button of tkmacosx which 
looks and feels exactly like a native tkinter button can change its background 
and foreground colors.

Read more about tkmacosx in detail on
https://github.com/Saadmairaj/tkmacosx#tkmacosx
"""

__version__ = '1.0.2'
__author__ = 'Saad Mairaj'

from tkmacosx.variables import ColorVar, DictVar, SaveVar
from tkmacosx.widgets import Button, CircleButton, Colorscale, Radiobutton, Marquee, SFrame
from tkmacosx.utils import (check_appearance, check_light_dark,
                            delta, hex_to_rgb, get_shade, gradient)



#                       Copyright 2021 Saad Mairaj
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import tkinter
from tkmacosx.utils import (_cnfmerge, _bind)

RADIOBUTTON_PROPERTIES = [
    'activebackground', 'activeforeground', 'anchor',
    'background', 'bitmap', 'borderwidth', 'command', 'compound',
    'cursor', 'disabledforeground', 'font', 'foreground', 'height',
    'highlightbackground', 'highlightcolor', 'highlightthickness',
    'image', 'indicatoron', 'justify', 'offrelief', 'overrelief',
    'padx', 'pady', 'relief', 'selectcolor', 'selectimage', 'state',
    'takefocus', 'text', 'textvariable', 'tristateimage', 'tristatevalue',
    'underline', 'value', 'variable', 'width', 'wraplength'
]


RADIOBUTTON_TEXT_PROPERTIES = (
    'activebackground',
    'activeforeground',
    'anchor',
    'background',
    'bd',
    'bg',
    'cursor',
    'disabledforeground',
    'fg',
    'font',
    'foreground',
    'justify',
    'state',
    'text',
    'textvariable',
    'underline',
    'wraplength'
)


class _radiobutton_functions:

    def _check_setit(self, kw={}):
        """Internal function.\n
        Checks if indicator patch needed or not."""
        cond1 = kw.get('indicator', kw.get('indicatoron', self['indicator']))
        if (not cond1 and
                not (kw.get('bitmap', self['bitmap']) or
                     kw.get('image', self['image']))):
            return True
        return False

    def _if_selected(self):
        """Internal function.\n
        Checks if the radiobutton is selected or not."""
        if self._cnf['variable'] is not None:
            if (isinstance(self._cnf['variable'], tkinter.StringVar)
                    and isinstance(self['value'], int)):
                return self._cnf['variable'].get() == str(self['value'])
            return self._cnf['variable'].get() == self['value']
        return True

    def _change_selector_color(self, wid, mode):
        """Internal function."""
        lb = getattr(self, '_indi_lb', False)
        if not lb or not lb.winfo_exists():
            return
        if mode == 'checked':
            self._indi_lb['bg'] = self._cnf['selectcolor']
            if wid != self:
                tkinter.Radiobutton._configure(
                    self, 'configure', {'selectcolor': self._cnf['selectcolor']}, None)
            return
        self._indi_lb['bg'] = self['bg']
        if wid != self:
            tkinter.Radiobutton._configure(
                self, 'configure', {'selectcolor': self['bg']}, None)

    def _bind_handler(self, *ags, **kw):
        """Internet function.\n
        Don't call this function directly."""
        _bind(self, *ags, **kw)
        lb = getattr(self, '_indi_lb', False)
        if lb and lb.winfo_exists():
            _bind(lb, *ags, **kw)

    def _internal_configure(self, **kw):
        """Internal function.\n
        Don't call this function directly. 
        Configure both radiobutton and the label."""
        tkinter.Radiobutton._configure(self, 'configure', kw, None)
        lb = getattr(self, '_indi_lb', False)
        if lb and lb.winfo_exists():
            lb._configure('configure', kw, None)

    def _on_press(self, evt=None):
        """Internal function. \n
        When widget is pressed <Button-1>."""
        def cmd(evt):
            self._bind_handler(className='button_command',
                               sequence='<ButtonRelease-1>')
            self._state('on_release')
            self._clear_unchecked()
            self._change_selector_color(evt.widget, 'checked')
            self.tk.call(self._w, 'select')
            if evt.widget != self:
                self.invoke()

        def on_enter(evt):
            """Internal function.\n
            Enables when pressed and cursor
            is moved back on widget."""
            self._state('on_enter')
            if self._if_selected():
                self._change_selector_color(evt.widget, 'unchecked')
            self._bind_handler(className='button_command',
                               sequence='<ButtonRelease-1>', func=cmd)

        def on_leave(evt):
            """Internal function.\n
            Disables/Cancels when pressed 
            and cursor is moved away from 
            the widget."""
            self._state('on_leave')
            if self._if_selected():
                self._change_selector_color(evt.widget, 'checked')
            self._bind_handler(className='button_command',
                               sequence='<ButtonRelease-1>')

        if self['state'] not in 'disabled':
            self._state('on_press')
            self._change_selector_color(evt.widget, 'unchecked')
            self._bind_handler(
                {'className': 'on_press_enter',
                    'sequence': '<Enter>', 'func': on_enter},
                {'className': 'on_press_leave',
                    'sequence': '<Leave>', 'func': on_leave},
                {'className': 'button_command',
                    'sequence': '<ButtonRelease-1>', 'func': cmd}
            )

    def _on_release(self, evt):
        """Internal function. \n
        When button is released <ButtonRelease-1>."""
        if self['state'] not in 'disabled':
            self._bind_handler(
                {'className': 'on_press_enter', 'sequence': '<Enter>'},
                {'className': 'on_press_leave', 'sequence': '<Leave>'},
                {'className': 'button_command', 'sequence': '<ButtonRelease-1>'}
            )

    def _clear_unchecked(self, evt=None):
        """Internal function."""
        for i in self._radiobuttons:
            lb = getattr(i, '_indi_lb', False)
            if (lb and lb.winfo_exists()
                and i._cnf['variable'] == self._cnf['variable']
                    and i != self and self._cnf['variable']):
                lb['bg'] = i['bg']

    def _state(self, mode):
        """Internal function."""
        if mode in ('on_enter', 'on_press'):
            self._internal_configure(
                state='active',
                background=self['activebackground'],
                foreground=self['activeforeground'],
            )
        elif mode in ('on_leave', 'on_release'):
            self._internal_configure(
                state='normal',
                background=self._cnf['background'],
                foreground=self._cnf['foreground']
            )

    def _setit(self):
        """Internal function.
        Sets the radio button with the patch."""
        self.propagate(0)
        self._indi_lb = tkinter.Label(self)
        for i in RADIOBUTTON_TEXT_PROPERTIES:
            self._indi_lb[i] = self[i]
        self._indi_lb.pack(expand=1, fill='both')

        if self._if_selected():
            self._indi_lb['bg'] = self['selectcolor']

        self._bind_handler(
            {'className': "on_press", "sequence": "<Button-1>"},  # Unbinds if any
            {'className': "on_release", "sequence": "<ButtonRelease-1>"},  # Unbinds if any
            {'className': "on_press", "sequence": "<Button-1>", "func": self._on_press},
            {'className': "on_release", "sequence": "<ButtonRelease-1>",
                "func": self._on_release},
        )

    def _revert(self):
        """Internal function.\n
        Un-patch the radio button when `indicatoron` is set to 1."""
        self._indi_lb.destroy()
        self.propagate(1)

    def _set_configure(self, cnf={}):
        """Internal function."""
        self._trace_fix(cnf)
        for p in list(self._cnf):
            self._cnf[p] = cnf.get(p, self._cnf[p])
        lb = getattr(self, '_indi_lb', False)
        if self._check_setit(cnf):
            if lb and lb.winfo_exists():
                for i in cnf:
                    if i in RADIOBUTTON_TEXT_PROPERTIES:
                        self._indi_lb[i] = cnf[i]
            else:
                self._setit()
        elif lb:
            self._revert()

    def _trace_fix(self, kw={}):
        """Internal function."""
        for i in ('background', 'foreground', 'fg', 'bg'):
            if kw.get(i) and isinstance(kw.get(i), tkinter.Variable):
                key = 'background' if i[0] == 'b' else 'foreground'
                kw[i].trace_add(
                    'write', lambda *a, var=kw[i], k=key:
                        self._cnf.update({k: var.get()}))


class RadiobuttonBase(tkinter.Radiobutton, _radiobutton_functions):
    _radiobuttons = []

    def __init__(self, master=None, cnf={}, **kw):
        kw = _cnfmerge((kw, cnf))
        self._trace_fix(kw)
        tkinter.Radiobutton.__init__(self, master, kw)
        self._radiobuttons.append(self)
        self._cnf = dict(
            variable=kw.get('variable'),
            selectcolor=self['selectcolor'],
            foreground=self['foreground'],
            background=self['background'],
        )

        # Set the patch if indicatoron is set to 0.
        if self._check_setit():
            self._setit()

        self._bind_handler(
            {'className': "on_press", "sequence": "<Button-1>", "func": self._on_press},
            {'className': "on_release", "sequence": "<ButtonRelease-1>",
                "func": self._on_release},
        )

    def _configure(self, cmd, cnf, kw):
        """Internal function."""
        self._set_configure(_cnfmerge((cnf, kw)))
        return super()._configure(cmd, cnf, kw)

    def deselect(self):
        """Put the button in off-state."""
        self._change_selector_color(None, 'unchecked')
        return super().deselect()

    def select(self):
        """Put the button in on-state."""
        self._clear_unchecked()
        self._change_selector_color(None, 'checked')
        return super().select()



#                       Copyright 2021 Saad Mairaj
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import ast
import tkinter


class DictVar(tkinter.Variable):
    """
    #### Value holder for Dictionaries.
    Get a specific value by getting the key from this \
    `get(self, key=None, d=None)` method if exists in the dictionary. \n
    if `key=None` it will return the complete dictionary.
    """
    _default = {}

    def __init__(self, master=None, value=None, name=None):
        """Construct a string variable.

        MASTER can be given as master widget.
        VALUE is an optional value (defaults to {})
        NAME is an optional Tcl name (defaults to PY_VARnum).

        If NAME matches an existing variable and VALUE is omitted
        then the existing value is retained.
        """
        tkinter.Variable.__init__(self, master, value, name)

    def get(self, key=None, d=None):
        """Return value of variable as string."""
        value = self._tk.globalgetvar(self._name)
        if not isinstance(value, dict):
            value = ast.literal_eval(value)
        if key:
            return value.get(key, d)
        return value



#                       Copyright 2021 Saad Mairaj
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""This file contains patches to support ColorVar with 
tkinter widgets and canvas items. Patches are decorators 
later used on tkmacosx widgets to support ColorVar."""

import tkinter


# {(self, option, tag): (var, cbname)}
_all_traces_colorvar = {}

_properties1 = (
    'activefill',
    'activeoutline',
    'disabledfill',
    'disabledoutline',
    'fill',
    'outline',
    'background',
    'activebackground',
    'activeforeground',
    'disabledbackground',
    'disabledforeground',
    'foreground'
)

_properties2 = (
    'fg',
    'foreground',
    'bg',
    'background',
    'activebackground',
    'activeforeground',
    'disabledforeground',
    'highlightbackground',
    'selectcolor',
    'highlightcolor',
    'selectforeground',
    'readonlybackground',
    'selectbackground',
    'insertbackground',
    'disabledbackground')


def _colorvar_patch_destroy(fn):
    """Internal function.\n
    Deletes the traces if any when widget is destroy."""

    def _patch(self):
        """Interanl function."""
        if self._tclCommands is not None:
            # Deletes the widget from the _all_traces_colorvar
            # and deletes the traces too.
            for key, value in dict(_all_traces_colorvar).items():
                if self == key[0]:
                    var, cbname = value
                    try:
                        var.trace_vdelete('w', cbname)
                    except tkinter.TclError:
                        pass
                    _all_traces_colorvar.pop(key)
        return fn(self)
    return _patch


def _colorvar_patch_configure(fn):
    """Internal function.\n
    Patch for Canvas items to support ColorVar functionality."""

    def _patch(self, cmd, cnf, kw):
        """Internal function."""
        if isinstance(cmd, tuple) and isinstance(self, tkinter.Canvas):
            tags = self.find('withtag', cmd[1])
            for tag in tags:
                for i in _properties1:
                    tmp_dict = cnf if cnf and i in cnf else kw
                    cnf_copy = dict(**tmp_dict) if tmp_dict else {}
                    if isinstance(cnf_copy.get(i), tkinter.Variable):
                        var = cnf_copy[i]
                        cbname = var.trace_variable(
                            'w', lambda a, b, c, cls=self, opt=i,
                            tagId=tag, var=var: cls._configure(
                                ('itemconfigure', tagId), {opt: var.get()}, None))
                        if (self, (i, tag)) in _all_traces_colorvar:
                            v, cb = _all_traces_colorvar.get((self, (i, tag)))
                            v.trace_vdelete('w', cb)
                            _all_traces_colorvar[(self, (i, tag))] = (
                                var, cbname)
                        else:
                            _all_traces_colorvar[(self, (i, tag))] = (
                                var, cbname)
                        tmp_dict[i] = var.get()
        return fn(self, cmd, cnf, kw)
    return _patch


def _colorvar_patch_options(fn):
    """Internal function.\n
    Patch for ColorVar to work with tkinter widgets."""

    def _patch(self, cnf, kw=None):
        """Internal function."""
        cnf = tkinter._cnfmerge(cnf)
        if kw:
            cnf = tkinter._cnfmerge((cnf, kw))

        for i in _properties2:
            if isinstance(cnf.get(i), tkinter.Variable):
                var = cnf[i]
                cbname = var.trace_variable(
                    'w', lambda a, b, c, i=i, var=var,
                    cls=self: cls.config({i: var.get()}))
                if (self, i) in _all_traces_colorvar:
                    v, cb = _all_traces_colorvar.get((self, i))
                    v.trace_vdelete('w', cb)
                    _all_traces_colorvar[(self, i)] = (var, cbname)
                else:
                    _all_traces_colorvar[(self, i)] = (var, cbname)
                cnf[i] = var.get()
        return fn(self, cnf, None)
    return _patch


def _create(self, itemType, args, kw):
    """Internal function."""
    args = tkinter._flatten(args)
    cnf = args[-1]
    if isinstance(cnf, (dict, tuple)):
        args = args[:-1]
    else:
        cnf = {}

    # Add the resources to the list to have ColorVar functionality.
    ckw = tkinter._cnfmerge((cnf, kw))
    var = None
    for i in _properties1:
        if isinstance(ckw.get(i), tkinter.Variable):
            var = ckw[i]
            _all_traces_colorvar[(self, (i, None))] = (var, None)
            if isinstance(cnf, dict) and i in cnf:
                cnf[i] = var.get()
            elif i in kw:
                kw[i] = var.get()

    # get unique tag id
    tagId = self.tk.getint(self.tk.call(
        self._w, 'create', itemType,
        *(args + self._options(cnf, kw))))

    for key, value in dict(_all_traces_colorvar).items():
        if isinstance(key[1], (tuple, list)):
            _, (opt, tag_id) = key
            var, cbname = value
            if tag_id is None and cbname is None:
                cbname = var.trace_variable(
                    'w', lambda a, b, c, cls=self, opt=opt,
                    tagId=tagId, var=var: cls._configure(
                        ('itemconfigure', tagId), {opt: var.get()}, None))
                _all_traces_colorvar[(self, (opt, tagId))] = (var, cbname)
                _all_traces_colorvar.pop((self, (opt, None)))
    return tagId


# initializing decorators here.
def patch():
    """Patch tkinter.Misc._options, tkinter.Misc.destroy, 
    tkinter.Misc._configure, tkinter.Canvas._create."""
    tkinter.Misc.destroy = _colorvar_patch_destroy(tkinter.Misc.destroy)
    tkinter.Misc._options = _colorvar_patch_options(tkinter.Misc._options)
    tkinter.Misc._configure = _colorvar_patch_configure(
        tkinter.Misc._configure)
    tkinter.Canvas._create = _create



from tkmacosx import __version__, __author__
from setuptools import setup, find_packages


def get_long_description(path):
    """Opens and fetches text of long descrition file."""
    with open(path, 'r') as f:
        text = f.read()
    return text


attrs = dict(
    name='tkmacosx',
    version=__version__,
    packages=find_packages(exclude=('test',)),
    long_description=get_long_description('README.md'),
    description='Tkmacosx is a Python library extension to the Tkinter module that let you change background color of the button on macOS.',
    long_description_content_type='text/markdown',
    author=__author__,
    author_email='Saadmairaj@yahoo.in',
    url='https://github.com/Saadmairaj/tkmacosx',
    license='Apache',
    python_requires='>=3',
    install_requires=[
        'colour',
    ],
    keywords=[
        'tkinter',
        'macos',
        'variable',
        'widgets',
        'colorscale',
        'tk',
        'color',
        'button',
        'circlebutton',
        'marquee',
        'tkinter-widgets',
        'tkinter-scrollableframe',
        'scrollableframe',
    ],
    classifiers=[
        'Development Status :: 6 - Mature',
        'Environment :: MacOS X',
        'Environment :: Win32 (MS Windows)',
        'Environment :: X11 Applications',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: End Users/Desktop',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.0',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Software Development',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Widget Sets',
        'License :: OSI Approved :: Apache Software License',
    ],
    project_urls={
        'Bug Reports': 'https://github.com/Saadmairaj/tkmacosx/issues',
        'Source': 'https://github.com/Saadmairaj/tkmacosx',
        'Documentation': 'https://github.com/Saadmairaj/tkmacosx#documentation',
    },
    include_package_data=True,
)

setup(**attrs)



import unittest
import tkinter
import tkmacosx
import tkinter.ttk as ttk

from test.widget_tests import (add_standard_options, noconv,
                               StandardOptionsTests, ButtonOptionsTests,
                               IntegerSizeTests, PixelSizeTests,
                               AbstractWidgetTest)


def float_round(x):
    return float(round(x))


class AbstractToplevelTest(AbstractWidgetTest, PixelSizeTests):
    _conv_pad_pixels = noconv

    def test_configure_class(self):
        widget = self.create()
        self.assertEqual(widget['class'],
                         widget.__class__.__name__.title())
        self.checkInvalidParam(widget, 'class', 'Foo',
                               errmsg="can't modify -class option after widget is created")
        widget2 = self.create(class_='Foo')
        self.assertEqual(widget2['class'], 'Foo')

    def test_configure_colormap(self):
        widget = self.create()
        self.assertEqual(widget['colormap'], '')
        self.checkInvalidParam(widget, 'colormap', 'new',
                               errmsg="can't modify -colormap option after widget is created")
        widget2 = self.create(colormap='new')
        self.assertEqual(widget2['colormap'], 'new')

    def test_configure_container(self):
        widget = self.create()
        self.assertEqual(widget['container'], 0 if self.wantobjects else '0')
        self.checkInvalidParam(widget, 'container', 1,
                               errmsg="can't modify -container option after widget is created")
        widget2 = self.create(container=True)
        self.assertEqual(widget2['container'], 1 if self.wantobjects else '1')

    def test_configure_visual(self):
        widget = self.create()
        self.assertEqual(widget['visual'], '')
        self.checkInvalidParam(widget, 'visual', 'default',
                               errmsg="can't modify -visual option after widget is created")
        widget2 = self.create(visual='default')
        self.assertEqual(widget2['visual'], 'default')


class AbstractLabelTest(AbstractWidgetTest, IntegerSizeTests):
    _conv_pixels = noconv

    def test_configure_highlightthickness(self):
        widget = self.create()
        self.checkPixelsParam(widget, 'highlightthickness',
                              0, 1.3, 2.6, 6, -2, '10p')


class AbstractButtonTest(AbstractWidgetTest, PixelSizeTests):

    _ttk_parent = False
    _ttk_parent_with_style = False
    _ttk_style = "TFrame"
    _type = 'normal'

    def create(self, **kwargs):
        self._master = self.root
        if self._ttk_parent and self._ttk_parent_with_style:
            style = ttk.Style(self.root)
            self._ttk_style = "Custom.TFrame"
            style.configure(self._ttk_style, background="pink")
            self._master = ttk.Frame(self.root, style=self._ttk_style)
            self._master.pack()
        elif self._ttk_parent:
            self._master = ttk.Frame(self.root)
            self._master.pack()
        if self._type == 'normal':
            return tkmacosx.Button(self._master, **kwargs)
        if self._type == 'circle':
            return tkmacosx.CircleButton(self._master, **kwargs)

    def test_configure_state(self):
        widget = self.create()
        self.checkEnumParam(widget, 'state', 'active',
                            'disabled', 'normal', 'pressed')

    def test_configure_highlightbackground(self):
        widget = self.create()
        self.checkColorParam(widget, 'highlightbackground')

        widget['borderless'] = True
        for c in ('#ff0000', '#00ff00', '#0000ff', '#123456',
                  'red', 'green', 'blue', 'white', 'black', 'grey'):
            widget['highlightbackground'] = c
            self.assertNotEqual(widget['highlightbackground'], c)

        widget['borderless'] = False
        for c in ('#ff0000', '#00ff00', '#0000ff', '#123456',
                  'red', 'green', 'blue', 'white', 'black', 'grey'):
            widget['highlightbackground'] = c
            self.assertEqual(widget['highlightbackground'], c)


@add_standard_options(StandardOptionsTests, ButtonOptionsTests)
class ButtonTest(AbstractButtonTest, unittest.TestCase):
    OPTIONS = (
        'activebackground', 'activeforeground', 'activeimage',
        'activebitmap', 'anchor', 'background', 'bitmap', 'bordercolor',
        'borderless', 'borderwidth', 'command', 'compound', 'cursor', 'disabledbackground',
        'disabledforeground', 'font', 'focuscolor', 'focusthickness',
        'foreground', 'height', 'highlightbackground', 'highlightcolor',
        'highlightthickness', 'image', 'justify', 'overbackground',
        'overforeground', 'overrelief', 'padx', 'pady', 'relief',
        'repeatdelay', 'repeatinterval', 'state', 'takefocus', 'text',
        'textvariable', 'underline', 'width')

    _conv_pixels = round
    _ttk_parent = False
    _ttk_parent_with_style = False
    _type = 'normal'


@add_standard_options(StandardOptionsTests, ButtonOptionsTests)
class Button_ttk_Test(AbstractButtonTest, unittest.TestCase):
    OPTIONS = (
        'bordercolor', 'borderless', 'highlightbackground',
        'highlightcolor', 'highlightthickness')

    _conv_pixels = round
    _ttk_parent = True


@add_standard_options(StandardOptionsTests, ButtonOptionsTests)
class Button_ttk_with_style_Test(AbstractButtonTest, unittest.TestCase):
    OPTIONS = (
        'bordercolor', 'borderless', 'highlightbackground',
        'highlightcolor', 'highlightthickness')

    _conv_pixels = round
    _ttk_parent = True
    _ttk_parent_with_style = True


@add_standard_options(StandardOptionsTests, ButtonOptionsTests)
class CircleButtonTest(AbstractButtonTest, unittest.TestCase):
    OPTIONS = (
        'activebackground', 'activeforeground', 'activeimage',
        'activebitmap', 'anchor', 'background', 'bitmap', 'bordercolor',
        'borderless', 'borderwidth', 'command', 'compound', 'cursor', 'disabledbackground',
        'disabledforeground', 'font', 'focuscolor', 'focusthickness',
        'foreground', 'height', 'highlightbackground', 'highlightcolor',
        'highlightthickness', 'image', 'justify', 'overbackground',
        'overforeground', 'overrelief', 'padx', 'pady', 'relief', 'radius',
        'repeatdelay', 'repeatinterval', 'state', 'takefocus', 'text',
        'textvariable', 'underline', 'width')

    _conv_pixels = round
    _type = 'circle'

    def test_configure_radius(self):
        widget = self.create()
        self.checkIntegerParam(widget, 'radius', 402, -402, 0)


@add_standard_options(StandardOptionsTests, ButtonOptionsTests)
class CircleButton_ttk_Test(AbstractButtonTest, unittest.TestCase):
    OPTIONS = (
        'bordercolor', 'borderless', 'highlightbackground',
        'highlightcolor', 'highlightthickness')

    _conv_pixels = round
    _ttk_parent = True
    _type = 'circle'


@add_standard_options(StandardOptionsTests, ButtonOptionsTests)
class CirclButton_ttk_with_style_Test(AbstractButtonTest, unittest.TestCase):
    OPTIONS = (
        'bordercolor', 'borderless', 'highlightbackground',
        'highlightcolor', 'highlightthickness')

    _conv_pixels = round
    _ttk_parent = True
    _ttk_parent_with_style = True
    _type = 'circle'


@add_standard_options(StandardOptionsTests, PixelSizeTests)
class ColorscaleTest(AbstractWidgetTest, unittest.TestCase):
    OPTIONS = (
        'borderwidth', 'cursor', 'command', 'gradient', 'height',
        'highlightbackground', 'highlightcolor', 'highlightthickness',
        'mousewheel', 'orient', 'relief', 'showinfo', 'showinfodelay',
        'state', 'takefocus', 'value', 'variable', 'width',
    )
    _conv_pixels = round
    _keep_orig = False
    default_orient = 'vertical'

    def create(self, **kwargs):
        return tkmacosx.Colorscale(self.root, **kwargs)

    def test_configure_gradient(self):
        widget = self.create()
        self.checkParams(
            widget, 'gradient', ('red', 'black'), 'default', ('#333', '#fff'))
        self.checkInvalidParam(widget, 'gradient', 'spam',
                               errmsg='expected sequence of two color values but got "spam"')
        self.checkInvalidParam(widget, 'gradient', ('spam', 'red'),
                               errmsg='unknown color name "spam"')
        self.checkInvalidParam(widget, 'gradient', ('#ff3123', 'spam'),
                               errmsg='unknown color name "spam"')

    def test_configure_mousewheel(self):
        widget = self.create()
        self.checkBooleanParam(widget, 'mousewheel')

    def test_configure_showinfo(self):
        widget = self.create()
        self.checkBooleanParam(widget, 'showinfo')

    def test_configure_state(self):
        widget = self.create()
        self.checkEnumParam(widget, 'state', 'disabled', 'normal',
                            errmsg='bad state value "{}": must be normal or disabled')

    def test_configure_showinfodelay(self):
        widget = self.create()
        self.checkIntegerParam(widget, 'showinfodelay', 0, 100, -100)

    def test_configure_value(self):
        widget = self.create()
        self.checkEnumParam(widget, 'value', 'rgb', 'hex')

    def test_configure_variable(self):
        widget = self.create()
        var = tkmacosx.ColorVar(self.root)
        self.checkVariableParam(widget, 'variable', var)


@add_standard_options(StandardOptionsTests, PixelSizeTests)
class MarqueeTest(AbstractWidgetTest, unittest.TestCase):
    OPTIONS = (
        'background', 'borderwidth', 'cursor', 'disabledforeground',
        'end_delay', 'foreground', 'font', 'fps', 'height', 'highlightbackground',
        'highlightcolor', 'highlightthickness', 'initial_delay', 'justify',
        'left_margin', 'relief', 'smoothness', 'state', 'takefocus', 'text', 'width',
    )

    _conv_pixels = round
    _stringify = True
    _keep_orig = False

    def create(self, **kwargs):
        kwargs['bg'] = kwargs.get('bg', kwargs.get('background', 'black'))
        kwargs['fg'] = kwargs.get('fg', kwargs.get('foreground', 'white'))
        return tkmacosx.Marquee(self.root, **kwargs)

    def test_configure_state(self):
        widget = self.create()
        self.checkEnumParam(widget, 'state', 'disabled', 'normal',
                            errmsg='bad state value "{}": must be normal or disabled')

    def test_configure_end_delay(self):
        widget = self.create()
        self.checkIntegerParam(widget, 'end_delay', 0, 100, '10')

    def test_configure_fps(self):
        widget = self.create()
        self.checkIntegerParam(widget, 'fps', 30, 60, 120, '30')

    def test_configure_initial_delay(self):
        widget = self.create()
        self.checkIntegerParam(widget, 'initial_delay', 0, 100, '10')

    def test_configure_left_margin(self):
        widget = self.create()
        self.checkIntegerParam(widget, 'left_margin', 0, 100, -100, '20')

    def test_configure_smoothness(self):
        widget = self.create()
        self.checkIntegerParam(widget, 'smoothness', 0, 1, 5, 6, -4, '20')

    def test_reset(self):
        widget = self.create()
        widget['text'] = 'Testing text functions'*4
        coords = widget.coords('text')
        widget['initial_delay'] = 0
        widget['end_delay'] = 0
        widget.reset()
        self.assertEqual(coords, widget.coords('text'))

    def test_play(self):
        widget = self.create()
        widget['text'] = 'Testing text functions'*4
        widget.stop(True)
        initial_coords = widget.coords('text')
        widget['initial_delay'] = 0
        widget['end_delay'] = 0
        coords = widget.coords('text')
        widget.play()
        self.assertNotEqual(coords, widget.coords('text'))
        widget.play(True)
        coords = widget.coords('text')
        coords[0] += 1
        self.assertEqual(initial_coords, coords)

    def test_stop(self):
        widget = self.create()
        widget['text'] = 'Testing text functions'*4
        initial_coords = widget.coords('text')
        widget['initial_delay'] = 0
        widget['end_delay'] = 0
        coords = widget.coords('text')
        widget.stop()
        self.assertEqual(coords, widget.coords('text'))
        widget.stop(True)
        self.assertEqual(initial_coords, coords)


@add_standard_options(StandardOptionsTests)
class RadiobuttonTest(AbstractLabelTest, unittest.TestCase):
    OPTIONS = (
        'activebackground', 'activeforeground', 'anchor',
        'background', 'bitmap', 'borderwidth', 'command', 'compound',
        'cursor', 'disabledforeground', 'font', 'foreground', 'height',
        'highlightbackground', 'highlightcolor', 'highlightthickness',
        'image', 'indicatoron', 'justify', 'offrelief', 'overrelief',
        'padx', 'pady', 'relief', 'selectcolor', 'selectimage', 'state',
        'takefocus', 'text', 'textvariable', 'tristateimage', 'tristatevalue',
        'underline', 'value', 'variable', 'width', 'wraplength'
    )

    def create(self, **kwargs):
        return tkmacosx.Radiobutton(self.root, **kwargs)

    def test_configure_value(self):
        widget = self.create()
        self.checkParams(widget, 'value', 1, 2.3, '', 'any string')


@add_standard_options(StandardOptionsTests)
class SFrameTest(AbstractToplevelTest, unittest.TestCase):
    OPTIONS = (
        'avoidmousewheel', 'autohidescrollbar', 'autohidescrollbardelay',
        'background', 'borderwidth', 'canvas', 'class', 'colormap', 'container',
        'cursor', 'height', 'highlightbackground', 'highlightcolor', 'highlightthickness',
        'mousewheel', 'padx', 'pady', 'relief', 'scrollbar', 'scrollbarwidth', 'takefocus',
        'visual', 'width'
    )

    def create(self, **kwargs):
        return tkmacosx.SFrame(self.root, **kwargs)

    def test_configure_autohidescrollbar(self):
        widget = self.create()
        self.checkBooleanParam(widget, 'autohidescrollbar')

    def test_configure_autohidescrollbardelay(self):
        widget = self.create()
        self.checkIntegerParam(
            widget, 'autohidescrollbardelay', 0, 100, 500, '1000')

    def test_configure_avoidmousewheel(self):
        widget = self.create()
        widget_list = tuple([tkmacosx.Colorscale(widget) for i in range(10)])
        self.checkParam(widget, 'avoidmousewheel', widget_list[0])
        widget['avoidmousewheel'] = widget_list
        for w in widget_list:
            self.assertIn(w, widget['avoidmousewheel'])

    def test_configure_canvas(self):
        widget = self.create()
        canvas = tkinter.Canvas(widget)
        wrong_widget = tkinter.Frame(widget)
        self.checkParam(widget, 'canvas', canvas)
        self.checkInvalidParam(widget, 'canvas', wrong_widget,
                               errmsg='expected tkinter.Canvas instance but got "%s"'
                               % wrong_widget)
        self.checkInvalidParam(widget, 'canvas', 10,
                               errmsg='expected tkinter.Canvas instance but got "10"')
        self.checkInvalidParam(widget, 'canvas', 'spam',
                               errmsg='expected tkinter.Canvas instance but got "spam"')

    def test_configure_mousewheel(self):
        widget = self.create()
        self.checkBooleanParam(widget, 'mousewheel')

    def test_configure_scrollbar(self):
        widget = self.create()
        scrollbar = tkinter.Scrollbar(widget)
        wrong_widget = tkinter.Frame(widget)
        self.checkParam(widget, 'scrollbar', scrollbar)
        self.checkInvalidParam(widget, 'scrollbar', wrong_widget,
                               errmsg='expected tkinter.Scrollbar instance but got "%s"'
                               % wrong_widget)
        self.checkInvalidParam(widget, 'scrollbar', 10,
                               errmsg='expected tkinter.Scrollbar instance but got "10"')
        self.checkInvalidParam(widget, 'scrollbar', 'spam',
                               errmsg='expected tkinter.Scrollbar instance but got "spam"')

    def test_configure_scrollbarwidth(self):
        widget = self.create()
        self.checkPixelsParam(
            widget, 'scrollbarwidth', 402, 403.4, 404.6, -402, 0, '5i')



import os
import pickle
import unittest
from tkinter import TclError, Tk
from tkmacosx import ColorVar, DictVar, SaveVar
from test.support import findfile


class TestBase(unittest.TestCase):

    _filename = findfile('testSaveVar.pkl')

    def checkSavefile(self, var, filename):
        with open(filename, 'rb') as file:
            d = pickle.load(file)
            if d.get(str(var)):
                return d[str(var)]

    def remove_file(self):
        if os.path.exists(self._filename):
            os.remove(self._filename)

    def setUp(self):
        self.remove_file()
        self.root = Tk()

    def tearDown(self):
        del self.root
        self.remove_file()


class TestColorVar(TestBase):

    def test_default(self):
        v = ColorVar(self.root)
        self.assertEqual("white", v.get())

    def test_get(self):
        v = ColorVar(self.root, "#333", "name")
        self.assertEqual("#333", v.get())
        self.root.globalsetvar("name", "#ff0000")
        self.assertEqual("#ff0000", v.get())

    def test_invalid_value(self):
        v = ColorVar(self.root, "#00ff00", "name")
        self.root.globalsetvar("name", "value")
        with self.assertRaises((ValueError, TclError)):
            v.get()


class TestDictVar(TestBase):

    def test_default(self):
        v = DictVar(self.root)
        self.assertEqual({}, v.get())

    def test_get(self):
        v = DictVar(self.root, {'1': 'abc'}, 'name')
        self.assertEqual({'1': 'abc'}, v.get())
        self.root.globalsetvar("name", {2: 'xyz'})
        self.assertEqual({2: 'xyz'}, v.get())


class TestSaveVar(TestBase):
    _default = 'blue'

    def _create(self, **kw):
        return SaveVar(
            var=kw.get('var', ColorVar),
            master=kw.get('master', self.root),
            value=kw.get('value', self._default),
            name=kw.get('name', 'name'),
            filename=kw.get('filename', self._filename))

    def test_default(self):
        v = self._create()
        self.assertEqual(
            self._default, self.checkSavefile(v, self._filename)[0])
        self.assertEqual(
            self._default, self.checkSavefile(v, self._filename)[1])

    def test_saved_value(self):
        v = self._create()
        v.set('#15234d')
        self.assertEqual("#15234d", v.get())
        self.assertEqual(v.get(), self.checkSavefile(v, self._filename)[0])
        self.assertEqual(
            self._default, self.checkSavefile(v, self._filename)[1])

    def test_file_created(self):
        v = self._create()
        file = open(self._filename, 'r')



#                       Copyright 2021 Saad Mairaj
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import tkinter

# patch to make tkinter.Variable with multiple options of widgets.
from tkmacosx.utils.colorvar_patches import patch
patch()


class ColorVar(tkinter.Variable):
    """Value holder for HEX color. Default is white"""

    _default = "white"

    def __init__(self, master=None, value=None, name=None):
        """Construct a color variable. (bg, fg, ..)

        MASTER can be given as master widget.
        VALUE is an optional value (defaults to "")
        NAME is an optional Tcl name (defaults to PY_VARnum).

        If NAME matches an existing variable and VALUE is omitted
        then the existing value is retained.
        """
        tkinter.Variable.__init__(self, master, value, name)

    def set(self, value):
        """Set the variable to VALUE."""
        self._root.winfo_rgb(value)
        return self._tk.globalsetvar(self._name, value)
    initialize = set

    def get(self):
        """Return value of variable color."""
        value = self._tk.globalgetvar(self._name)
        self._root.winfo_rgb(value)
        return str(value)
